#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

"""
Module of the Gantlet Testing Framework
"""

import events
from result_base import result_base, Part
import gts_tparser


class gts_result(result_base):
    """
    The results of a test emitting Gantlet Testing Syntax (GTS)

    Methods of this class are called by gts test as it parses the
    test's output.  The two classes coordinate closely by passing
    a state back and forth.
    """
    
    def __init__( self, raw_name, scoreboard, type ):
        """
        Create the test
        """
        self.declared_nparts = None
        result_base.__init__( self, raw_name, scoreboard, type )
        return

    def test_begin( self, state, test_begin ):
        """
        Invoked by gts_tparser when it parses a new test
        """
        if state != 'between tests' and state != 'before tests' and state != 'broken' :
            # if we're in the wrong state, flag as broken
            msg = "TEST_BEGIN encountered in \"%s\" state." % state
            self.broken( msg, 'state error')
            return 'broken'
        self.number = self.scoreboard.get_id()
        try:
            testname = test_begin.group('testname')
            # this is a named test, possibly a multitest
            if testname[0] == '/':
                self.name = self.raw_name + testname
            else:
                self.name = "%s[%s]" % ( self.raw_name, testname )
        except IndexError:
            pass
        self.declared_nparts = int(test_begin.group('nparts'))
        self.n_parts = self.declared_nparts # assume its correct (for now)
        self.scoreboard.event['test_begin'].publish( events.test_begin_event, self )
        return 'before parts'


    def part_begin( self, state, part_begin ):
        """
        Invoked by gts_tparser when it parses a part begin
        """
        partno = int(part_begin.group('partno'))
        if (state == 'in part') and (partno == len(self.part)+1):
            msg = "Missing \"RESULT %d\" statement: set to UNRESOLVED" % (partno-1,)
            self.warning(msg,'RESULT %d missing' % (partno-1,))
            part_end = gts_tparser._part_end.search( "RESULT %d UNRESOLVED" % (partno-1,))
            self.part_end( state, part_end )
        elif state != 'between parts' and state != 'before parts':
            msg = "PART %d encountered in \"%s\" state." % (partno, state)
            self.broken( msg, 'state error')
            return 'broken'        
        part = Part()
        part.number = partno
        self.part.append(part)
        self.scoreboard.event['part_begin'].publish( events.part_begin_event, (self,partno) )
        return 'in part'

    def part_end( self, state, part_end ):
        """
        Invoked by gts_tparser when it parses a parts end        
        """
        resultno = int(part_end.group('partno'))
        if  state != 'in part':
            msg = "result %d encountered in \"%s\" state." % (resultno, state)
            self.broken( msg, 'state error')
            return 'broken'
        part = self.part[-1]
        if resultno != part.number: 
            msg = "part begin #%d != result #%d." % (part.number, resultno)
            self.broken( msg, 'state error')
            return 'broken'
        part.result = part_end.group('result')
        self.part[-1] = part
        self.scoreboard.event['part_end'].publish( events.part_end_event, (self, resultno) )
        return 'between parts'

    def test_end( self, state, test_end ):
        """
        Invoked by gts_tparser when it parses the end of a test
        """
        if not self.exit_status and test_end.group('exitstatus'):
            self.exit_status = int( test_end.group('exitstatus') )
        if self.exit_status and not self.is_broken:
            msg = "Abnormal Termination %d" % self.exit_status
            canon = "exit_status=%d" % self.exit_status
            self.broken( msg, canon )
        if state == 'before parts':
            msg = "No \"PART\" Directive found"
            self.broken( msg, 'parse error')
        elif state != 'between parts':
            msg = "test end encountered in \"%s\" state." % state
            self.broken( msg, 'state error')
        self.declared_result = test_end.group('result')
        self.PTC_confirm_nparts()
        self.PTC_confirm_result()
        self.scoreboard.event['test_end'].publish( events.test_end_event, self )
        return 'between tests'

    def try_recover_premature_end( self, state ):
        """
        Sometimes tests just don't end where you wnat them.
        And ya gotta go back and figure out why.
        """
        if state == 'before tests': # declare its begin and end
            test_begin = gts_tparser._test_begin.search('TEST_BEGIN 0')
            self.test_begin( state, test_begin)
            msg = "No \"TEST_BEGIN\" Directive found"
            self.broken( msg, 'parse error')
            if self.exit_status:
                msg = "Abnormal Temination %d" % self.exit_status
                self.broken( msg, 'exit_status=%d' % self.exit_status )
            test_end = gts_tparser._test_end.search('TEST_RESULT BROKEN')
            self.test_end('between parts', test_end )
        elif state == 'before parts': # missing part declaration
            msg = "No \"PART\" Directive found"
            self.broken( msg, 'parse error')
            if self.exit_status:
                msg = "Abnormal Temination %d" % self.exit_status
                self.broken( msg, "exit_status=%d" % self.exit_status )
            test_end = gts_tparser._test_end.search('TEST_RESULT BROKEN')
            self.test_end('between parts', test_end )
        elif state == 'between parts': # missing test end event
            if self.exit_status:
                msg = "Abnormal Temination %d" % self.exit_status
                self.broken( msg, "exit_status=%d" % self.exit_status )
            if ( self.n_parts == len( self.part ) ):
                msg = "No \"TEST_RESULT\" Directive found"
                self.warning( msg, 'TEST_RESULT missing')
            else:
                msg = "No \"TEST_RESULT\" Directive found"
                self.broken( msg, 'parse error')
            test_end = gts_tparser._test_end.search('TEST_RESULT FAIL')
            self.test_end('between parts', test_end )
        elif state == 'in part': # missing part end and test end
            partno = len(self.part)
            msg = "Missing \"RESULT %d\" statement: set to UNRESOLVED" % (partno,)
            self.warning(msg,'RESULT %d missing' % (partno,))
            part_end = gts_tparser._part_end.search( "RESULT %d UNRESOLVED" % (partno,))
            self.part_end( state, part_end )
            msg = "No \"TEST_RESULT\" Directive found"
            self.broken( msg, 'parse error')
            if self.exit_status:
                msg = "Abnormal Temination %d in part %d of %d" % ( self.exit_status, len(self.part), self.n_parts)
                self.broken(  msg, 'exit_status=%d'% self.exit_status )
            self.scoreboard.event['test_end'].publish( events.test_end_event, self )     
        elif state == 'broken':
            self.scoreboard.event['test_end'].publish( events.test_end_event, self )     
        else:
            msg = "test ended in state \"%s\"" % state
            self.broken( msg, 'premature end')




#
# File:          Cstring_Impl.py
# Symbol:        Strings.Cstring-v1.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class Strings.Cstring in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Class to allow testing of string passing using every possible mode.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import Strings.Cstring
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
import string
# DO-NOT-DELETE splicer.end(_before_type)

class Cstring:
  """\
Class to allow testing of string passing using every possible mode.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = Strings.Cstring.Cstring(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def returnback(self, nonNull):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # bool nonNull
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

    """\
If <code>nonNull</code> is <code>true</code>, this will
return \"Three\"; otherwise, it will return a NULL or empty string.
"""
# DO-NOT-DELETE splicer.begin(returnback)
    if (nonNull):
      return "Three"
    else:
      return None
# DO-NOT-DELETE splicer.end(returnback)

  def passin(self, c):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # string c
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

    """\
This will return <code>true</code> iff <code>c</code> equals \"Three\".
"""
# DO-NOT-DELETE splicer.begin(passin)
    return c == "Three"
# DO-NOT-DELETE splicer.end(passin)

  def passout(self, nonNull):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # bool nonNull
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, c)
    # bool _return
    # string c
    #

    """\
If <code>nonNull</code> is <code>true</code>, this will return
\"Three\" in <code>c</code>; otherwise, it will return a null or
empty string. The return value is <code>false</code> iff 
the outgoing value of <code>c</code> is <code>null</code>.
"""
# DO-NOT-DELETE splicer.begin(passout)
    if (nonNull):
      return 1, "Three"
    else:
      return 0, None
# DO-NOT-DELETE splicer.end(passout)

  def passinout(self, c):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # string c
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, c)
    # bool _return
    # string c
    #

# DO-NOT-DELETE splicer.begin(passinout)
    if (c and (len(c) > 0)):
      if ((c[0] >= "a") and (c[0] <= "z")):
        c = string.upper(c[0]) + c[1:]
      else:
        if ((c[0] >= "A") and (c[0] <= "Z")):
          c = string.lower(c[0]) + c[1:]
      c = c + "s"
      return 1, c
    else:
      return 0, c
# DO-NOT-DELETE splicer.end(passinout)

  def passeverywhere(self, c1, c3):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # string c1
    # string c3
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, c2, c3)
    # string _return
    # string c2
    # string c3
    #

# DO-NOT-DELETE splicer.begin(passeverywhere)
    result = None
    if (c3):
      if (len(c3) > 0):
        if ((c3[0] >= "a") and (c3[0] <= "z")):
          c3 = string.upper(c3[0]) + c3[1:]
        else:
          if ((c3[0] >= "A") and (c3[0] <= "Z")):
            c3 = string.lower(c3[0]) + c3[1:]
        c3 = c3[:-1]
      if (c1 == "Three"):
        result = "Three"
    return result, "Three", c3
# DO-NOT-DELETE splicer.end(passeverywhere)

  def mixedarguments(self, s1, c1, s2, c2):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # string s1
    # char c1
    # string s2
    # char c2
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

# DO-NOT-DELETE splicer.begin(mixedarguments)
    return (s1 == s2) and (c1 == c2)
# DO-NOT-DELETE splicer.end(mixedarguments)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          MergeSort_Impl.py
# Symbol:        sort.MergeSort-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.MergeSort in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Merge sort
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.Comparator
import sort.Container
import sort.Counter
import sort.MergeSort
import sort.SortingAlgorithm
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
def mergeLists(elems, comp, cmp, swp, start, mid, end):
  while ((start < mid) and (mid < end)):
    cmp.inc()
    if (elems.compare(start, mid, comp) > 0):
      for j in xrange(mid, start, -1):
        swp.inc()
        elems.swap(j, j - 1)
      mid = mid + 1
    start = start + 1

def mergeSort(elems, comp, cmp, swp, start, end):
  if ((end - start) > 1):
    mid = (start + end) >> 1
    mergeSort(elems, comp, cmp, swp, start, mid)
    mergeSort(elems, comp, cmp, swp, mid, end)
    mergeLists(elems, comp, cmp, swp, start, mid, end)
    
# DO-NOT-DELETE splicer.end(_before_type)

class MergeSort:
  """\
Merge sort
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.MergeSort.MergeSort(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def sort(self, elems, comp):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # sort.Container elems
    # sort.Comparator comp
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Sort elements using Merge Sort.
"""
# DO-NOT-DELETE splicer.begin(sort)
    size = elems.getLength()
    cmp = self.__IORself.getCompareCounter()
    swp = self.__IORself.getSwapCounter()
    mergeSort(elems, comp, cmp, swp, 0, size)
# DO-NOT-DELETE splicer.end(sort)

  def getName(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

    """\
Return merge sort.
"""
# DO-NOT-DELETE splicer.begin(getName)
    return "Merge sort"
# DO-NOT-DELETE splicer.end(getName)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          Basic_Impl.py
# Symbol:        objarg.Basic-v0.5
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class objarg.Basic in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
from types import NoneType
# DO-NOT-DELETE splicer.end(_initial)

import objarg.Basic
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
# Insert-Code-Here {_before_type} ()
# DO-NOT-DELETE splicer.end(_before_type)

class Basic:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = objarg.Basic.Basic(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
# Insert-Code-Here {__init__} ()
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def passIn(self, o, inNotNull):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # sidl.BaseClass o
    # bool inNotNull
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

    """\
Return inNotNull == (o != NULL).
"""
# DO-NOT-DELETE splicer.begin(passIn)
    return (inNotNull and not isinstance(o,NoneType)) or not (inNotNull or not isinstance(o,NoneType))
# DO-NOT-DELETE splicer.end(passIn)

  def passInOut(self, o, inNotNull, outNotNull, retSame):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # sidl.BaseClass o
    # bool inNotNull
    # bool outNotNull
    # bool retSame
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, o)
    # bool _return
    # sidl.BaseClass o
    #

    """\
Return inNotNull == (o != NULL).  If outNotNull, the outgoing
value of o should not be NULL; otherwise, it will be NULL.
If outNotNull is true, there are two cases, it retSame is true
the incoming value of o will be returned; otherwise, a new
object will be allocated and returned.
"""
# DO-NOT-DELETE splicer.begin(passInOut)
    retval = (inNotNull and not isinstance(o,NoneType)) or not (inNotNull or not isinstance(o,NoneType))
    if outNotNull:
      if not retSame or isinstance(o, NoneType):
        o = sidl.BaseClass.BaseClass()
    else:
      o = None
    return (retval, o)
# DO-NOT-DELETE splicer.end(passInOut)

  def passOut(self, passOutNull):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # bool passOutNull
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # sidl.BaseClass o
    #

    """\
If passOutNull is true, a NULL value of o will be returned; otherwise,
a newly allocated object will be returned.
"""
# DO-NOT-DELETE splicer.begin(passOut)
    if (passOutNull):
      return None
    else:
      return sidl.BaseClass.BaseClass()
# DO-NOT-DELETE splicer.end(passOut)

  def retObject(self, retNull):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # bool retNull
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # sidl.BaseClass _return
    #

    """\
Return a NULL or non-NULL object depending on the value of retNull.
"""
# DO-NOT-DELETE splicer.begin(retObject)
    if (retNull):
      return None
    else:
      return sidl.BaseClass.BaseClass()
# DO-NOT-DELETE splicer.end(retObject)

# DO-NOT-DELETE splicer.begin(_final)
# Insert-Code-Here {_final} ()
# DO-NOT-DELETE splicer.end(_final)



#
# File:          QuickSort_Impl.py
# Symbol:        sort.QuickSort-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.QuickSort in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Quick sort
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.Comparator
import sort.Container
import sort.Counter
import sort.QuickSort
import sort.SortingAlgorithm
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
def choosePivot(elems, comp, cmp, start, end):
  pivot = (start + end) >> 1
  if ((end - start) > 4):
    mid = pivot
    cmp.inc()
    if (elems.compare(start, mid, comp) <= 0):
      cmp.inc()
      if (elems.compare(mid, end - 1, comp) > 0):
        cmp.inc()
        if (elems.compare(start, end - 1, comp) < 0):
          pivot = end - 1
        else:
          pivot = start
    else:
      cmp.inc()
      if (elems.compare(mid, end - 1, comp) < 0):
        cmp.inc()
        if (elems.compare(start, end - 1, comp) > 0):
          pivot = end - 1
        else:
          pivot = start
  return pivot

def quickSort(elems, comp, cmp, swp, start, end):
  if ((end - start) > 1):
    pivot = choosePivot(elems, comp, cmp, start, end)
    i = start
    j = end
    if (pivot != start):
      swp.inc()
      elems.swap(start, pivot)
    while 1:
      j = j - 1
      cmp.inc()
      while (elems.compare(start, j, comp) < 0):
        j = j - 1
        cmp.inc()
      i = i + 1
      while (i < j):
        cmp.inc()
        if (elems.compare(start, i, comp) < 0): break
        i = i + 1
      if (i >= j): break
      swp.inc()
      elems.swap(i,j)
    if (j != start):
      swp.inc()
      elems.swap(start, j)
    quickSort(elems, comp, cmp, swp, start, j)
    quickSort(elems, comp, cmp, swp, j+1, end)
      
# DO-NOT-DELETE splicer.end(_before_type)

class QuickSort:
  """\
Quick sort
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.QuickSort.QuickSort(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def sort(self, elems, comp):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # sort.Container elems
    # sort.Comparator comp
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Sort elements using Quick Sort.
"""
# DO-NOT-DELETE splicer.begin(sort)
    size = elems.getLength()
    cmp = self.__IORself.getCompareCounter()
    swp = self.__IORself.getSwapCounter()
    quickSort(elems, comp, cmp, swp, 0, size)
# DO-NOT-DELETE splicer.end(sort)

  def getName(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

    """\
Return quick sort.
"""
# DO-NOT-DELETE splicer.begin(getName)
    return "Quick sort"
# DO-NOT-DELETE splicer.end(getName)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.
import sys
import string
import xml.sax
import xml.sax.handler
import gantlet.events
from gantlet.unknown_tparser import unknown_tparser
from gantlet.xml_result import xml_result
from gantlet.tparser_base import tparser_base

class xml_tparser (  xml.sax.handler.ContentHandler, tparser_base ):
    """
    This class parses an XML file containing multiple tests
    """
    def __init__( self, invocation, scoreboard ):
        tparser_base.__init__(self, invocation, scoreboard )
        self.state = []          # a stack to keep track of state
        self.buffer = []    # buffer to accumulate character data
        self.tests = []          # a queue of xml_result instances
        self.n_promised_tests = 1

    def startElement( self, name, attrs ):
        self.PVT_flush_buffer()
        self.state.append( name )
        if name == 'test':
            self.tests.append( xml_result(self.invocation, self.scoreboard ) )
            if len(self.tests) > self.n_promised_tests:
                diff = len(self.tests) - self.n_promised_tests
                self.n_promised_tests = len(self.tests)
                self.scoreboard.event["test_added"].publish( \
                    gantlet.events.test_added_event, diff )                
        if name == 'attribute':
            self.attributes( attrs )
        elif name in ( 'test', 'part', 'error', 'warning', 'comment'):
            start_element[name]( self.tests[-1], attrs )

    def endElement( self, name ):        
        if name != self.state[-1]:
            print "error ending element\"%s\" but expected \"%s\"" % \
                  ( name, self.state[-1] )
        self.state.pop()
        if name in ('test', 'part'):
            self.PVT_flush_buffer()
            end_element[ name ]( self.tests[-1] )
        elif name in ('error', 'warning', 'comment'):
            msg = string.join(self.buffer,"").strip()
            end_element[ name ]( self.tests[-1], msg )
            self.buffer = []
        else:
            self.PVT_flush_buffer()
                    
    def attributes( self, attrs ):
        if attrs['key'] == 'total_tests':
            value = int(attrs['value'])
            self.n_promised_tests = value
            self.scoreboard.event["test_added"].publish( \
                    gantlet.events.test_added_event, value-1 )
        
    def characters( self, content ): 
        self.buffer.append(content)

    def runtest( self ):
        try:
            tparser = xml.sax.make_tparser()
        except:
            #create bogus test and mark it broken
            self.invocation.errmsgs = [ ( str(sys.exc_info()[0]), \
                                          "missing SAX") ]
            t = unknown_tparser(self.invocation, self.scoreboard ) 
            self.tests.append( t )
            t.runtest()
        else:
            tparser.setContentHandler( self )
            tparser.parse( self.invocation.filename )

            

    def PVT_flush_buffer(self):
        if len ( self.buffer ):
            s = string.join( self.buffer,"" ).strip()
            if len (s):
                self.scoreboard.event['text'].publish( \
                    gantlet.events.text_event, s)
            self.buffer= []
            
                                    

def xml_extension( ex ):
    if ( ex == '.xml' or ex == '.XML' ):
        return 1
    else:
        return 0



start_element = {}
start_element['gantlet'] = xml_result.PVT_start_stub
start_element['head'] = xml_result.PVT_start_stub
start_element['attribute'] = xml_result.PVT_start_stub
start_element['body'] = xml_result.PVT_start_stub
start_element['test'] = xml_result.test_begin
start_element['part'] = xml_result.part_begin
start_element['comment'] = xml_result.comment_begin
start_element['warning'] = xml_result.warning_begin
start_element['error'] = xml_result.error_begin

end_element = {}
end_element['gantlet'] = xml_result.PVT_end_stub
end_element['head'] = xml_result.PVT_end_stub
end_element['attribute'] = xml_result.PVT_end_stub
end_element['body'] = xml_result.PVT_end_stub
end_element['test'] = xml_result.test_end
end_element['part'] = xml_result.part_end
end_element['comment'] = xml_result.comment_end
end_element['warning'] = xml_result.warning_end
end_element['error'] = xml_result.error_end

if __name__ == '__main__':
    print 'hi'



#
# File:          J_Impl.py
# Symbol:        Inherit.J-v1.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class Inherit.J in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import Inherit.A
import Inherit.B
import Inherit.C
import Inherit.E2
import Inherit.J
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_type)

class J:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = Inherit.J.J(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def j(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

# DO-NOT-DELETE splicer.begin(j)
    # Put your code here...
    return "J.j"
# DO-NOT-DELETE splicer.end(j)

  def e(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

# DO-NOT-DELETE splicer.begin(e)
    # Put your code here...
    return "J.E2.e"
# DO-NOT-DELETE splicer.end(e)

  def c(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

# DO-NOT-DELETE splicer.begin(c)
    # Put your code here...
    return "J.E2.c"
# DO-NOT-DELETE splicer.end(c)

  def a(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

# DO-NOT-DELETE splicer.begin(a)
    # Put your code here...
    return "J.a"
# DO-NOT-DELETE splicer.end(a)

  def b(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

# DO-NOT-DELETE splicer.begin(b)
    # Put your code here...
    return "J.b"
# DO-NOT-DELETE splicer.end(b)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          CompInt_Impl.py
# Symbol:        sort.CompInt-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.CompInt in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""Compare two Integer's.  By default, this will sort in increasing order.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.CompInt
import sort.Comparator
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
import sort.Integer
# DO-NOT-DELETE splicer.end(_before_type)

class CompInt:
  """\
Compare two Integer's.  By default, this will sort in increasing order.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.CompInt.CompInt(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    self.d_increasing = 1
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def setSortIncreasing(self, increasing):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # bool increasing
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
If increasing is true, this will cause the comparator to
report a normal definition of less than; otherwise, it will
reverse the normal ordering.
"""
# DO-NOT-DELETE splicer.begin(setSortIncreasing)
    self.d_increasing = increasing
# DO-NOT-DELETE splicer.end(setSortIncreasing)

  def compare(self, i1, i2):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # sidl.BaseInterface i1
    # sidl.BaseInterface i2
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # int _return
    #

    """\
This method is used to define an ordering of objects.  This method
will return -1 if i1 < i2, 0 if i1 = i2; and 1 if i1 > i2.
"""
# DO-NOT-DELETE splicer.begin(compare)
    result = 0
    int1 = sort.Integer.Integer(i1)
    int2 = sort.Integer.Integer(i2)
    if (int1 and int2):
      if (int1.getValue() > int2.getValue()):
        result = 1
      if (int1. getValue() < int2.getValue()):
        result = -1
      if (not self.d_increasing):
        result = -result
    return result
# DO-NOT-DELETE splicer.end(compare)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

"""
Contains the events and event handlers used in Gantlet
"""

class event_base:
    """
    Base class for all events
    """
    def __init__( self, method ):
        """
        Create with hook for method to call
        """
	self.method = method
        return

    def fire( self, data ):
        """
        Fire predetermined method with data tuple
        """
	self.method( data )
        return

class test_begin_event( event_base ):
    pass

class test_end_event( event_base ):
    pass

class test_added_event( event_base ):
    pass

class part_begin_event( event_base ):
    pass

class part_end_event( event_base ):
    pass

class test_interrupt_event( event_base ):
    pass

class test_exitstatus_event( event_base ):
    pass

class test_warning_event( event_base ):
    pass

class test_error_event( event_base ):
    pass

class suite_finish_event( event_base ):
    pass

class text_event( event_base ):
    pass

class comment_event( event_base ):
    pass

class event_handler:
    """
    Generic handler for all events
    """
    def __init__( self, event_type ):
        """
        Create a handler for a specific event type

        event_type must be a subclass of event_base
        """
	assert issubclass(event_type, event_base), 'type must inherit from event_base'
        self.event_type = event_type
        self.subscribers = []
        return

    def subscribe( self, event ):
        """
        subscribe a new event listener 
        """
        if isinstance( event, self.event_type ):
            self.subscribers.append(event)
            return 1
        else:
            return 0

    def publish( self, event_type, data ):
        """
        calls fire on all matching subscribed events
        """
        if issubclass( event_type, self.event_type ):
            for e in self.subscribers:
                e.fire(data)
            return 1
        else:
            return 0

		
def _test():
    """
    Simple test driver.
    """
    def myprint1( data ):
	print "1 ", data

    def myprint2( data ):
	print "2 ", data

    def myprint3( data ):
	print "3 ", data

    def myprint4( data ):
	print "4 ", data

    class test_begin_event2(test_begin_event):
	pass

    print "ready to start"
    x = event_handler( test_begin_event )
    print "subscribe"
    results = { 
      "parent" : x.subscribe( event_base( myprint1 ) ),
      "self" : x.subscribe( test_begin_event( myprint2 ) ),
      "sibling" : x.subscribe( test_end_event( myprint3 ) ),
      "child" : x.subscribe( test_begin_event2( myprint4 ) )
    }
    print results

    print "publish"
    tuple = [
      x.publish( event_base, "parent" ),
      x.publish( test_begin_event, "self"),
      x.publish( test_end_event, "sibling"),
      x.publish( test_begin_event2, "child")
    ]
    print tuple
    return

if __name__ == '__main__':
    _test()




#
# File:          __init__.py
# Symbol:        sidl-v0.9.17
# Symbol Type:   package
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Release:       $Name:  $
# Revision:      @(#) $Id: $
# Description:   package initialization code
# 
# Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by the Components Team <components@llnl.gov>
# All rights reserved.
# 
# This file is part of Babel. For more information, see
# http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
# for Our Notice and the LICENSE file for the GNU Lesser General Public
# License.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by
# the Free Software Foundation) version 2.1 dated February 1999.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
# conditions of the GNU Lesser General Public License for more details.
# 
# You should have recieved a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""The <code>sidl</code> package contains the fundamental type and interface
definitions for the <code>sidl</code> interface definition language.  It
defines common run-time libraries and common base classes and interfaces.
Every interface implicitly inherits from <code>sidl.BaseInterface</code>
and every class implicitly inherits from <code>sidl.BaseClass</code>.
"""

__all__ = [
   "BaseClass",
   "BaseException",
   "BaseInterface",
   "CastException",
   "ClassInfo",
   "ClassInfoI",
   "ClauseType",
   "ContractClass",
   "DFinder",
   "DLL",
   "EnfPolicy",
   "EnfTraceLevel",
   "EnforceFreq",
   "Finder",
   "InvViolation",
   "LangSpecificException",
   "Loader",
   "MemAllocException",
   "NotImplementedException",
   "PostViolation",
   "PreViolation",
   "Resolve",
   "RuntimeException",
   "SIDLException",
   "Scope",
   "io",
   "rmi" ]

try:
  from pkgutil import extend_path
  __path__ = extend_path(__path__, __name__)
except: # ignore all exceptions
  pass



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

"""
A module of the Gantlet Testing Framework
"""

import os

from gantlet.gts_tparser import gts_tparser

class gts_tparser_exec(gts_tparser):
   """
   A GTS tparsers specific to reading the output of executables.
   """
   
   def __init__( self, invocation, scoreboard ):
      """
      create an instance with an invocation object and scoreboard
      """
      gts_tparser.__init__(self, invocation, scoreboard )
      self.exit_status = 0

   def get_type( self ):
      """
      This can be redone with introspection
      """
      return "gts_tparser_exec"
      
   def runtest( self ):
      """
      fork the test and read its stderr and stdout
      """
      dirname = self.invocation.dirname
      basename = self.invocation.basename
      extension = self.invocation.extension
      switches = self.invocation.switches
      arguments = self.invocation.arguments
      
      if ( self.invocation.cd_to_file != None ):
         cd_to_file = self.invocation.cd_to_file
      elif not self.invocation.is_command:
         cd_to_file = self.scoreboard.config.cd_to_file
      else:
         cd_to_file = 1
      
      if cd_to_file:
         if dirname != "" and dirname != ".":
            command = "cd %s; ./%s%s %s %s" % ( dirname, basename, extension,
                                                switches, arguments)
            tmpfile = "%s/%s.testresult"%( dirname, basename)
         else:
            command = "./%s%s %s %s" % ( basename, extension,
                                            switches, arguments)
            tmpfile = "./%s.testresult"%( basename)
      else:
         if dirname != "" :
            command = "%s/%s%s %s %s" % ( dirname, basename, extension, switches, arguments )
            tmpfile = "%s/%s.testresult"%( dirname, basename)
         else:
            command = "%s%s %s %s" % ( basename, extension, switches, arguments )
            tmpfile = "./%s.testresult"%( basename)

      filehandle1 = os.popen( command, "r")
      filehandle2 = os.tmpfile()
      filehandle3 = open( tmpfile, "w");
      for line in filehandle1.xreadlines():
         filehandle2.write( line )
         filehandle3.write( line )
      self.exit_status = filehandle1.close()
      if self.exit_status:
         self.exit_status = self.exit_status >> 8
      filehandle2.seek(0)
      self.collect_results( filehandle2 )
      filehandle2.close()
      filehandle3.close()



#
# File:          vExcept_Impl.py
# Symbol:        vect.vExcept-v1.0
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6561M trunk)
# Description:   Implementation of sidl class vect.vExcept in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseException
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.SIDLException
import sidl.io.Deserializer
import sidl.io.Serializable
import sidl.io.Serializer
import vect.vExcept
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(_before_type)

class vExcept:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = vect.vExcept.vExcept(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

# DO-NOT-DELETE splicer.begin(_final)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(_final)



# Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
# Produced at the Lawrence Livermore National Laboratory.
# Written by the Components Team <components@llnl.gov>
# UCRL-CODE-2002-054
# All rights reserved.
# 
# This file is part of Babel. For more information, see
# http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
# for Our Notice and the LICENSE file for the GNU Lesser General Public
# License.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by
# the Free Software Foundation) version 2.1 dated February 1999.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
# conditions of the GNU Lesser General Public License for more details.
# 
# You should have recieved a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
all = [ "sidl", "sidlObjA", "sidlStruct", "sidlPyArrays" ]



#
# File:          Fib_Impl.py
# Symbol:        ExceptionTest.Fib-v1.0
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class ExceptionTest.Fib in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""This class holds the method <code>getFib</code> that generates the
requested Fibonacci numbers.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import ExceptionTest.Fib
import ExceptionTest.FibException
import ExceptionTest.NegativeValueException
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.MemAllocException
import sidl.RuntimeException
import sidl.SIDLException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
import ExceptionTest.TooDeepException
import ExceptionTest.TooBigException
# DO-NOT-DELETE splicer.end(_before_type)

class Fib:
  """\
This class holds the method <code>getFib</code> that generates the
requested Fibonacci numbers.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = ExceptionTest.Fib.Fib(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Put your code here...
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def getFib(self, n, max_depth, max_value, depth):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int n
    # int max_depth
    # int max_value
    # int depth
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # int _return
    #

    """\
<p>
Generate the requested Fibonacci number or generate exceptions if
the input Fibonacci number is invalid or if any of the maximum depth
or maximum value parameters are exceeded.  The last argument of the
method should be zero.
</p>
<p>
The algorithm should be similar to the <code>Java</code> code below.
</p>
<pre>
public int getFib(int n, int max_depth, int max_value, int depth)
throws NegativeValueException, FibException {

if (n < 0) {
throw new NegativeValueException(\"n negative\");

} else if (depth > max_depth) {
throw new TooDeepException(\"too deep\");

} else if (n == 0) {
return 1;

} else if (n == 1) {
return 1;

} else {
int a = getFib(n-1, max_depth, max_value, depth+1);
int b = getFib(n-2, max_depth, max_value, depth+1);
if (a + b > max_value) {
throw new TooBigException(\"too big\");
}
return a + b;
}
} 
</pre>
"""
# DO-NOT-DELETE splicer.begin(getFib)
    if (n < 0):
      ex = ExceptionTest.NegativeValueException.NegativeValueException()
      ex.setNote("n negative")
      ex.add(__name__, 0, "ExceptionTest.Fib.getFib")
      raise ExceptionTest.NegativeValueException._Exception, ex
    if (depth > max_depth):
      ex =  ExceptionTest.TooDeepException.TooDeepException()
      ex.setNote("too deep")
      ex.add(__name__, 0, "ExceptionTest.Fib.getFib")
      raise ExceptionTest.FibException._Exception, ex
    if (n == 0):
      return 1
    if (n == 1):
      return 1
    try:
      a = self.__IORself.getFib(n - 1, max_depth, max_value, depth + 1)
      b = self.__IORself.getFib(n - 2, max_depth, max_value, depth + 1)
      if (a + b > max_value):
        ex = ExceptionTest.TooBigException.TooBigException()
        ex.setNote("too big")
        ex.add(__name__, 0, "ExceptionTest.Fib.getFib")
        raise ExceptionTest.FibException._Exception, ex
      return a + b
    except ExceptionTest.FibException._Exception, exc:
      exc.exception.add(__name__, 0, "ExceptionTest.Fib.getFib")
      raise ExceptionTest.FibException._Exception, exc
# DO-NOT-DELETE splicer.end(getFib)

  def noLeak(self, a1, a2, r1, r2, c1, c2, s1, s2, o1, o2):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # array<int,2,column-major> a1
    # array<int,2,column-major> a2
    # rarray<int,2> r1
    # rarray<int,2> r2
    # array<int> c1
    # array<int> c2
    # string s1
    # string s2
    # sidl.BaseClass o1
    # sidl.BaseClass o2
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, a2, a3, r2, c2, c3, s2, s3, o2, o3)
    # array<int,2,column-major> _return
    # array<int,2,column-major> a2
    # array<int,2,column-major> a3
    # rarray<int,2> r2
    # array<int> c2
    # array<int> c3
    # string s2
    # string s3
    # sidl.BaseClass o2
    # sidl.BaseClass o3
    #

    """\
Check for memory/reference leaks in the presence of an exception.
The impl will throw an exception and assign random values to
out parameters to prove that out values are ignored.
The intent is that row-major arrays should be passed to parameters
a1, a2, a3.
"""
# DO-NOT-DELETE splicer.begin(noLeak)
    # Python requires no cleanup
    myException = sidl.SIDLException.SIDLException()
    myException.setNote("This method must throw an exception.")
    raise  sidl.SIDLException._Exception, myException
# DO-NOT-DELETE splicer.end(noLeak)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          Utils_Impl.py
# Symbol:        vect.Utils-v1.0
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6561M trunk)
# Description:   Implementation of sidl class vect.Utils in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.PostViolation
import sidl.PreViolation
import sidl.RuntimeException
import vect.Utils
import vect.vDivByZeroExcept
import vect.vNegValExcept
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_static)
import re
import math
import string
import vect.vExcept

import sidlPyArrays
if sidlPyArrays.type == "numpy":
  from numpy import zeros, float64, ndarray
  ArrayType = ndarray
elif sidlPyArrays.type == "numeric":
  import Numeric
  zeros = Numeric.zeros
  float64 = Numeric.Float64
  ArrayType = Numeric.ArrayType

#
# Some handy utilities from the arrays regression tests
def savespace(o):
  try:
    o.savespace(1)
  except AttributeError:
    pass

def typecode(o):
  try:
    return o.typecode()
  except AttributeError:
    return o.dtype.char

def createDouble(len):
  result = None
  if (len >= 0):
    result = zeros((len, ), float64)
    savespace(result)
  return result

def createDouble2d(len):
  result = None
  if (len >= 0):
    result = zeros((len, len, ), float64)
    savespace(result)
  return result

# DO-NOT-DELETE splicer.end(_before_static)

def vuIsZero(u, tol):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # double tol
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
boolean result operations 
Return TRUE if the specified vector is the zero vector, within the
given tolerance level; otherwise, return FALSE.
"""
# DO-NOT-DELETE splicer.begin(vuIsZero)
  isZ = 0

  if (isinstance(u, ArrayType) and (len(u.shape) == 1)):
    isZ = 1
    for i in range(u.shape[0]):
      if (math.fabs(u[i]) > math.fabs(tol)):
        isZ = 0
        break

  return isZ
# DO-NOT-DELETE splicer.end(vuIsZero)

def vuIsUnit(u, tol):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # double tol
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Return TRUE if the specified vector is the unit vector, within the
given tolerance level; otherwise, return FALSE.
"""
# DO-NOT-DELETE splicer.begin(vuIsUnit)
  isU = 0

  if ( isinstance(u, ArrayType) and (len(u.shape) == 1) ):
    if ( math.fabs(vuNorm(u, tol, 0) - 1.0) <= math.fabs(tol) ):
      isU = 1

  return isU
# DO-NOT-DELETE splicer.end(vuIsUnit)

def vuAreEqual(u, v, tol):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # double tol
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Return TRUE if the specified vectors are equal, within the given
tolerance level; otherwise, return FALSE.
"""
# DO-NOT-DELETE splicer.begin(vuAreEqual)
  areEq = 0

  if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
    if ( (len(u.shape) == 1) and (len(v.shape) == 1) ):
      if (u.shape[0] == v.shape[0]):
        areEq = 1
        for i in range(u.shape[0]):
          if (math.fabs(u[i] - v[i]) > math.fabs(tol)):
            areEq = 0
            break

  return areEq
# DO-NOT-DELETE splicer.end(vuAreEqual)

def vuAreOrth(u, v, tol):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # double tol
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Return TRUE if the specified vectors are orthogonal, within the given
tolerance; otherwise, return FALSE.
"""
# DO-NOT-DELETE splicer.begin(vuAreOrth)
  areOrth = 0

  if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
    absVal = math.fabs(vuDot(u, v, tol, 0))
    if (absVal <= math.fabs(tol)):
      areOrth = 1

  return areOrth
# DO-NOT-DELETE splicer.end(vuAreOrth)

def vuSchwarzHolds(u, v, tol):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # double tol
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Return TRUE if the Schwarz (or Cauchy-Schwarz) inequality holds, within
the given tolerance; otherwise, return FALSE.
"""
# DO-NOT-DELETE splicer.begin(vuSchwarzHolds)
  sHolds = 0

  if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
    if ( (len(u.shape) == 1) and (len(v.shape) == 1) ):
      if (u.shape[0] == v.shape[0]):
        absDot   = math.fabs(vuDot(u, v, tol, 0))
        absNorms = math.fabs(vuNorm(u, tol, 0) * vuNorm(v, tol, 0))
        if (absDot <= absNorms + math.fabs(tol)):
          sHolds = 1

  return sHolds
# DO-NOT-DELETE splicer.end(vuSchwarzHolds)

def vuTriIneqHolds(u, v, tol):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # double tol
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # bool _return
  #

  """\
Return TRUE if the Minkowski (or triangle) inequality holds, within the
given tolerance; otherwise, return FALSE.
"""
# DO-NOT-DELETE splicer.begin(vuTriIneqHolds)
  tiHolds = 0

  if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
    if ( (len(u.shape) == 1) and (len(v.shape) == 1) ):
      if (u.shape[0] == v.shape[0]):
        absNormSum  = math.fabs(vuNorm(vuSum(u,v,0), tol, 0))
        absSumNorms = math.fabs(vuNorm(u, tol, 0) + vuNorm(v, tol, 0))
        if (absNormSum <= absSumNorms + math.fabs(tol)):
          tiHolds = 1

  return tiHolds
# DO-NOT-DELETE splicer.end(vuTriIneqHolds)

def vuNorm(u, tol, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # double tol
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # double _return
  #

  """\
double result operations 
Return the norm (or length) of the specified vector.

Note that the size exception is given here simply because the
implementation is leveraging the vuDot() method.  Also the tolerance
is included to enable the caller to specify the tolerance used in
contract checking.

Also note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a negative result (regardless of input);
2 = Return a positive result not near zero (regardless of input), 
which means a violation will occur only if u is the zero
vector; and
3 = Return a zero result (regardless of input), which means a
violation will occur only if u is not the zero vector.
"""
# DO-NOT-DELETE splicer.begin(vuNorm)
  res = 0.0

  if (badLevel == 0):
    if (isinstance(u, ArrayType)):
      dot = vuDot(u, u, tol, 0)
      if (dot > 0.0):
        res = math.sqrt(dot)
      elif (dot < 0.0):
        res = -5.0
        exc =  vect.vNegValExcept.vNegValExcept()
        exc.setNote("vuNorm: vNegValExcept: Cannot sqrt() a negative value.")
        exc.add(__name__, 0, "vect.Utils.vuNorm")
        raise vect.vNegValExcept._Exception, exc
      else:
        res = 0.0
    else:
      res = 0.0
  elif (badLevel == 1):
    res = -5.0
  elif (badLevel == 2):
    res = 5.0
  elif (badLevel == 3):
    res = 0.0
  else:
    res = -5.0
  
  return res
# DO-NOT-DELETE splicer.end(vuNorm)

def vuDot(u, v, tol, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # double tol
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # double _return
  #

  """\
Return the dot (, inner, or scalar) product of the specified vectors.

Note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a negative result (regardless of input); and
2 = Return a positive result not near zero (regardless of input), 
which means a violation will occur only if u and v are the zero
vector.
"""
# DO-NOT-DELETE splicer.begin(vuDot)
  dot = 0.0

  if (badLevel == 0):
    if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
      if ( (len(u.shape) == 1) and (len(v.shape) == 1) ):
        if (u.shape[0] == v.shape[0]):
          for i in range(u.shape[0]):
            dot += u[i] * v[i]
        else:
          dot = 0.0
      else:
        dot = 0.0
    else:
      dot = 0.0
  elif (badLevel == 1):
    dot = -5.0
  elif (badLevel == 2):
    dot = 5.0
  elif (badLevel == 3):
    dot = 0.0
  else:
    dot = -5.0
  
  return dot
# DO-NOT-DELETE splicer.end(vuDot)

def vuProduct(a, u, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # double a
  # array<double> u
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> _return
  #

  """\
vector result operations 
Return the (scalar) product of the specified vector.

Note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a null result (regardless of input);
2 = Return a 2D array result (regardless of input); and
3 = Return an array of a different size (regardless of input).
"""
# DO-NOT-DELETE splicer.begin(vuProduct)
  prod = None

  if (isinstance(u, ArrayType)):
    lenU = u.shape[0]
  else:
    lenU = 0

  if (badLevel == 0):
    if (isinstance(u, ArrayType)):
      if (len(u.shape) == 1):
        prod = createDouble(lenU)
        for i in range(lenU):
          prod[i] = a * u[i]
      else:
        prod = None
    else:
      prod = None
  elif (badLevel == 1):
    prod = None
  elif (badLevel == 2):
    prod = createDouble2d(lenU)
  elif (badLevel == 3):
    prod = createDouble(lenU+5)
  else:
    prod = None
  
  return prod
# DO-NOT-DELETE splicer.end(vuProduct)

def vuNegate(u, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> _return
  #

  """\
Return the negation of the specified vector.

Note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a null result (regardless of input);
2 = Return a 2D array result (regardless of input); and
3 = Return an array of a different size (regardless of input).
"""
# DO-NOT-DELETE splicer.begin(vuNegate)
  negU = None

  if (isinstance(u, ArrayType)):
    lenU = u.shape[0]
  else:
    lenU = 0

  if (badLevel == 0):
    if (isinstance(u, ArrayType)):
      if (len(u.shape) == 1):
        negU = vuProduct(-1.0, u, 0)
      else:
        negU = None
    else:
      negU = None
  elif (badLevel == 1):
    negU = None
  elif (badLevel == 2):
    negU = createDouble2d(lenU)
  elif (badLevel == 3):
    negU = createDouble(lenU+5)
  else:
    negU = None
  
  return negU
# DO-NOT-DELETE splicer.end(vuNegate)

def vuNormalize(u, tol, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # double tol
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> _return
  #

  """\
Return the normalizaton of the specified vector.

Note the tolerance is included because the implementation invokes 
vuDot().

Note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a null result (regardless of input);
2 = Return a 2D array result (regardless of input); and
3 = Return an array of a different size (regardless of input).
"""
# DO-NOT-DELETE splicer.begin(vuNormalize)
  prod = None

  if (isinstance(u, ArrayType)):
    lenU = u.shape[0]
  else:
    lenU = 0

  if (badLevel == 0):
    if (isinstance(u, ArrayType)):
      if (len(u.shape) == 1):
        val = vuNorm(u, tol, 0)
        if (val != 0.0):
          prod = vuProduct(1.0/val, u, 0)
        else:
          prod = None
          exc  =  vect.vDivByZeroExcept.vDivByZeroExcept()
          exc.setNote("vuNormalize: vDivByZeroExcept: Cannot divide by zero.")
          exc.add(__name__, 0, "vect.Utils.vuNormalize")
          raise vect.vDivByZeroExcept._Exception, exc
      else:
        prod = None
    else:
      prod = None
  elif (badLevel == 1):
    prod = None
  elif (badLevel == 2):
    prod = createDouble2d(lenU)
  elif (badLevel == 3):
    prod = createDouble(lenU+5)
  else:
    prod = None
  
  return prod
# DO-NOT-DELETE splicer.end(vuNormalize)

def vuSum(u, v, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> _return
  #

  """\
Return the sum of the specified vectors.

Note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a null result (regardless of input);
2 = Return a 2D array result (regardless of input); and
3 = Return an array of a different size (regardless of input).
"""
# DO-NOT-DELETE splicer.begin(vuSum)
  aSum = None

  if (isinstance(u, ArrayType)):
    lenU = u.shape[0]
  else:
    lenU = 0

  if (badLevel == 0):
    if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
      if ( (len(u.shape) == 1) and (len(v.shape) == 1) ):
        if (lenU == v.shape[0]):
          aSum = createDouble(lenU)
          for i in range(lenU):
            aSum[i] = u[i] + v[i]
        else:
          aSum = None
      else:
        aSum = None
    else:
      aSum = None
  elif (badLevel == 1):
    aSum = None
  elif (badLevel == 2):
    aSum = createDouble2d(lenU)
  elif (badLevel == 3):
    aSum = createDouble(lenU+5)
  else:
    aSum = None
  
  return aSum
# DO-NOT-DELETE splicer.end(vuSum)

def vuDiff(u, v, badLevel):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # array<double> u
  # array<double> v
  # int badLevel
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # array<double> _return
  #

  """\
Return the difference of the specified vectors.

Note that badLevel has been added only to facilitate regression
testing of postconditions.  The levels are:
0 = NONE  (i.e., no deliberate postcondition violation);
1 = Return a null result (regardless of input);
2 = Return a 2D array result (regardless of input); and
3 = Return an array of a different size (regardless of input).
"""
# DO-NOT-DELETE splicer.begin(vuDiff)
  diff = None

  if (isinstance(u, ArrayType)):
    lenU = u.shape[0]
  else:
    lenU = 0

  if (badLevel == 0):
    if ( isinstance(u, ArrayType) and isinstance(v, ArrayType) ):
      if ( (len(u.shape) == 1) and (len(v.shape) == 1) ):
        if (lenU == v.shape[0]):
          diff = createDouble(lenU)
          for i in range(u.shape[0]):
            diff[i] = u[i] - v[i]
        else:
          diff = None
      else:
        diff = None
    else:
      diff = None
  elif (badLevel == 1):
    diff = None
  elif (badLevel == 2):
    diff = createDouble2d(lenU)
  elif (badLevel == 3):
    diff = createDouble(lenU+5)
  else:
    diff = None
  
  return diff
# DO-NOT-DELETE splicer.end(vuDiff)

# DO-NOT-DELETE splicer.begin(_before_type)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(_before_type)

class Utils:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = vect.Utils.Utils(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

# DO-NOT-DELETE splicer.begin(_final)
# Nothing needed here.
# DO-NOT-DELETE splicer.end(_final)



#
# File:          __init__.py
# Symbol:        sidlx.rmi-v0.1
# Symbol Type:   package
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Description:   package initialization code
# 
# WARNING: Automatically generated; changes will be lost
# 
#


""" 
"""

__all__ = [
   "BadFileDescriptorException",
   "CallType",
   "ChildSocket",
   "ClientSocket",
   "Common",
   "ConnectionRefusedException",
   "ConnectionResetException",
   "GenNetworkException",
   "IPv4Socket",
   "JimEchoServer",
   "NetworkUnreachableException",
   "NoServerException",
   "NotEnoughMemoryException",
   "OutOfAddressesException",
   "RecoverableException",
   "RetryException",
   "ServerSocket",
   "Settings",
   "SimCall",
   "SimHandle",
   "SimReturn",
   "SimpleOrb",
   "SimpleServer",
   "SimpleTicket",
   "SimpleTicketBook",
   "Simsponse",
   "Simvocation",
   "Socket",
   "Statistics",
   "TimeoutException",
   "TooManyOpenFilesException",
   "UnauthorizedCallException",
   "UnexpectedCloseException",
   "UnrecognizedNetworkException",
   "UnrecoverableException" ]

try:
  from pkgutil import extend_path
  __path__ = extend_path(__path__, __name__)
except: # ignore all exceptions
  pass



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.
"""
Entry point for running tests with Gantlet
"""

import sys
import getopt
import os.path
import re

import gantlet
#import gantlet.config
from scoreboard import scoreboard
from tparser_factory import tparser_factory
from classic_display import classic_display
from xml_display import xml_display
from email_display import email_display
from gui_display import gui_display

help_string="""
HELP:

 -h   | --help
 -q   | --quiet                   Suppresses classic display to screen
 -g   | --gui                     Enable GUI progress meter
 -s<> | --srcdir=<>               <>=directory, for VPATH builds
 -i<> | --histfile=<>             <>=filename, one line appended about results
 -c<> | --config=<>               <>=string, passed as argument to all tests
 -e   | --allow_exitstatus_only   
 -v<> | --verbose=<>              <>=integer, highernumbers more output
 -f<> | --testfile=<>             <>=filename, one test per line
 -f-  | --testfile=-                           reads from stdin instead of file
 -d   | --nowarn_deprecated       do not issue warnings about deprecated syntax
 -x<> | --xmlout=<>               <>=filename, produce XML output
 -m<> | --email=<>                <>=email address, email XML output
 -p<> | --profile=<>              <>=string, a distinguishing name associated with the suite
 -k<> | --package=<>              <>=string, name of the package you are testing
"""
        
def create_suite( argv ):
    """
    creates a gantlet.scoreboard initialized by the argument list
    returns a (scoreboard, remaining_args) tuple
    """
    shortopts = "qgs:hi:c:ev:f:dx:m:p:k:"
    longopts = ["quiet", "gui", "srcdir=", "help", "histfile=", "config=",
                "allow_exitstatus_only", "verbose=", "testfile=",
                "nowarn_deprecated","xmlout=", "email=","profile=","package="];

    (opts, args) = getopt.getopt( argv, shortopts, longopts )

    if len( args ) > 0:
        s = scoreboard(args[0])
    else:
        s = scoreboard('empty')

    for (o, v) in opts:
        if o in ("-h","--help"):
            print help_string
            sys.exit(0)
        elif o in ("-q", "--quiet"):
            s.config.quiet = 1
        elif o in ("-g", "--gui"):
            s.config.gui = 1
        elif o in ("-s", "--srcdir"):
            s.config.srcdir = v
        elif o in ("-i", "--histfile"):
            s.config.histfile = v
        elif o in ("-c", "--config"):
            s.config.name = v
        elif o in ("-e", "--allow_exitstatus_only"):
            s.config.allow_exit_status_only = 1
        elif o in ("-v", "--verbose"):
            s.config.verbose = int(v)
        elif o in ("-f", "--testfile"):
            if v == '-':
                file = stdin
            else:
                file = open(v,'r')
            args = args + gettests( file ) 
        elif o in ("-d", "--nowarn_deprecated"):
            s.config.warn_deprecated = 0
        elif o in ("-x", "--xmlout"):
            s.config.xmlout = v
        elif o in ("-m", "--email"):
            s.config.email = v
        elif o in ("-p", "--profile"):
            s.profile = v
        elif o in ("-k", "--package"):
            s.package = v
    return s, args[1:]

def expandvars( args ):
    """returns a matching argument list with paths and environment variables expanded"""
    result = [ os.path.expandvars( os.path.expanduser( arg ) ) for arg in args ]
    return result

def group_brackets( args ):
    """
       args = group_parens( args )

       groups a list of strings according to parens, e.g.
	['hi', 'there', '[', 'one', 'two', ']', 'hi', '[', 'three', 'four', ']', 'foo']
       to 
	['hi', 'there', ' one two', 'hi', ' three for', 'foo']

       Very useful after fighting with all kinds of yucky quoting rules in make.
       Does not handle nested brackets.
    """
    result = []
    temp = args[:]
    temp.reverse()
    while ( len(temp) != 0 ):
        arg = temp.pop()
        new_arg = ''
        if ( arg == '[' ):
            while( len(temp) != 0 ):
                arg = temp.pop()
                if ( arg == ']'):
                    break
                new_arg = new_arg + ' ' + arg 
        else:
            new_arg = arg
        result.append(new_arg.strip())
    return result

def gettests( file ):
    """
    tests[] = gettests( file );

    Parses input file.
    Strips '#' to end of line.
    Strips blank lines.
    """
    tests = []
    rough_tests = file.readlines()
    for line in rough_tests:
        line = re.split("#", line,1)[0]
        line = line.strip()
        if len(line):
            tests.append(line)
    return tests


def runall( s, tests ):
    """
    runs each test, then tells scoreboard to finish
    """
    for test in tests:
        # print test.brief_info()
        test.runtest()
        
    s.finish()
    return
    

def main():
    """
    the main entry point
    """
    (s, args) = create_suite( sys.argv[1:] )
    if not s.config.quiet:
        classic_display( s, sys.stdout )

    args = expandvars( args )
    args = group_brackets( args )        
    tf = tparser_factory( args, s )
    tests = tf.get_tests()
    s.event['test_added'].publish( gantlet.events.test_added_event, len(tests) )
    
    if s.config.xmlout :
        xml_display( s, open( s.config.xmlout,'w' ) )

    if s.config.email :
        email_display(s, s.config.email, s.config.smtpserver )

    if s.config.gui :
        gui = gui_display( s )
        gui.root.after( 1000, runall, s, tests )
        gui.root.mainloop()
    else:
        runall( s, tests )

    errcode = s.tot_tests_broken + s.tot_tests_failed
    if errcode:
        sys.stderr.write( \
            "Error: gantlet: %d failed tests and %d broken tests.\n"% \
            ( s.tot_tests_failed, s.tot_tests_broken ) )
        sys.exit( errcode )
    return
    
if __name__ == '__main__':
    main()




#
# File:          Basics_Impl.py
# Symbol:        hooks.Basics-v1.0
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6561M trunk)
# Description:   Implementation of sidl class hooks.Basics in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


# DO-NOT-DELETE splicer.begin(_initial)
# Insert-Code-Here {_initial} ()
# DO-NOT-DELETE splicer.end(_initial)

import hooks.Basics
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_static)
import synch
from synch import RegOut
from synch.ResultType import *
# DO-NOT-DELETE splicer.end(_before_static)

def aStaticMeth_pre(i, io):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int i
  # int io
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # None
  #

  """\
Basic illustration of hooks for static methods.
"""
# DO-NOT-DELETE splicer.begin(aStaticMeth_pre)
  #
  # This method has not been implemented
  #
  tracker = synch.RegOut.getInstance()
  tracker.startPart(1)
  return
# DO-NOT-DELETE splicer.end(aStaticMeth_pre)

def aStaticMeth(i, io):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int i
  # int io
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # (_return, o, io)
  # int _return
  # int o
  # int io
  #

  """\
Basic illustration of hooks for static methods.
"""
# DO-NOT-DELETE splicer.begin(aStaticMeth)
  #
  # This method has not been implemented
  #
  return (1,0,0);
# DO-NOT-DELETE splicer.end(aStaticMeth)

def aStaticMeth_post(i, o, io, _retval):
  #
  # sidl EXPECTED INCOMING TYPES
  # ============================
  # int i
  # int o
  # int io
  # int _retval
  #

  #
  # sidl EXPECTED RETURN VALUE(s)
  # =============================
  # None
  #

  """\
Basic illustration of hooks for static methods.
"""
# DO-NOT-DELETE splicer.begin(aStaticMeth_post)
  #
  # This method has not been implemented
  #
  tracker = synch.RegOut.getInstance()
  tracker.endPart(1, PASS)
  return
# DO-NOT-DELETE splicer.end(aStaticMeth_post)

# DO-NOT-DELETE splicer.begin(_before_type)
# Insert-Code-Here {_before_type} ()
# DO-NOT-DELETE splicer.end(_before_type)

class Basics:

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = hooks.Basics.Basics(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    # Insert-Code-Here {__init__} ()
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def aNonStaticMeth_pre(self, i, io):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int i
    # int io
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Basic illustration of hooks for static methods.
"""
# DO-NOT-DELETE splicer.begin(aNonStaticMeth_pre)
    #
    # This method has not been implemented
    #
    tracker = synch.RegOut.getInstance()
    tracker.startPart(2)
    return
# DO-NOT-DELETE splicer.end(aNonStaticMeth_pre)

  def aNonStaticMeth(self, i, io):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int i
    # int io
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # (_return, o, io)
    # int _return
    # int o
    # int io
    #

    """\
Basic illustration of hooks for static methods.
"""
# DO-NOT-DELETE splicer.begin(aNonStaticMeth)
    #
    # This method has not been implemented
    #
    return (1,0,0)
# DO-NOT-DELETE splicer.end(aNonStaticMeth)

  def aNonStaticMeth_post(self, i, o, io, _retval):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int i
    # int o
    # int io
    # int _retval
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Basic illustration of hooks for static methods.
"""
# DO-NOT-DELETE splicer.begin(aNonStaticMeth_post)
    #
    # This method has not been implemented
    #
    tracker = synch.RegOut.getInstance()
    tracker.endPart(2, PASS)
    return
# DO-NOT-DELETE splicer.end(aNonStaticMeth_post)

# DO-NOT-DELETE splicer.begin(_final)
# Insert-Code-Here {_final} ()
# DO-NOT-DELETE splicer.end(_final)



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

import re
import os.path
import cPickle
import gzip
import fcntl
from sys import maxint
from time import strptime, mktime
# from fcntl import LOCK_UN, LOCK_EX
from gantlet.session_email_parser import session_email_parser

class EntrySort:
    """ Sort based primarily on key, and secondarily on some other sort
    
    e.g.  EntrySort( 'package', EntrySort('profile', EntrySort('date')))
    sorts by package, then profile, then date attributes
    """

    def __init__(self, key, secondary=None):
        self.key = key
        self.secondary = secondary
    
    def set_secondary( self, secondary ):
        self.secondary = secondary
        
    def __call__( self, x, y ):
        if x[self.key] == y[self.key] and self.secondary:
            return self.secondary.__call__(x,y)
        else:
            return cmp( x[self.key], y[self.key] )
                   
class session_archiver:
    """Maintains an archive of Gantlet Messages

    Maintains a 'index.gantlet' file at the root of the archive.
    Will fetch read-only filehandles based on attibutes such as
    package, platform, config, date.  Files are stored in
    the repository as packagename/platformname/date-confix.

    """
    def __init__( self, root, create=None ):
        root = os.path.expanduser(root)
        root = os.path.expandvars(root)
        root = os.path.normpath(root)
        root = os.path.normcase(root)
        if not os.path.exists( root ):
            os.mkdir( root )
        if os.path.isdir(root):
            self.root = root
        self.index = []
        self.lockf = None
        if not create:
            self.load_index()


    def rescan(self):
        """Rescan directory and rebuild index"""
        os.path.walk( self.root, self._visit, self )
        return

    def _visit( self, foo, dirname, names ):
        """ Used in by self.rescan() to walk the repository
        """
        print self
        print foo
        for name in names:
            if name[-3:]=='.gz':
                print name
                f = gzip.open( os.path.join( dirname, name ), 'r' )
                s = session_email_parser( f, id=len(self.index) )
                p = self.createpath(s)
                entry = s.attr.copy()
                entry['path']=p
                self.index.append( entry )
                f.close()
        return

    def load_index(self):
	# self.lockf = open(  os.path.join( self.root, 'gantlet.index.lock' ), 'r' )
        # fcntl.flock(self.lockf.fileno(), LOCK_EX)
        f = open( os.path.join( self.root, 'gantlet.index' ), 'r' )
        p = cPickle.Unpickler( f )
        self.index = p.load()
	f.close()
        
    def store_index(self):
        f = open( os.path.join( self.root, 'gantlet.index' ), 'w' )
        p = cPickle.Pickler( f ) 
        p.dump( self.index )
        f.close()
        # if self.lockf:
        #     fcntl.flock(self.lockf.fileno(), LOCK_UN)
        #     self.lockf = None

    def select(self, package='.*', profile='.*', session='.*',
               starttime=0, endtime=maxint):
        """ get a list of entries corresponding to query of attributes via re
and a date range.
        """
        matches = []
        for entry in self.index:
            entrytime = mktime(strptime("%s %s" %
                                        (entry['date'], entry['time']),
                                        "%Y-%m-%d %H:%M:%S"))
            if ( re.match(package,entry['package']) and
                 re.match(profile,entry['profile']) and
                 re.match(session,entry['session']) and
                 (starttime <= entrytime <= endtime)):
                matches.append( entry )
        return matches
                
    def createpath( self, msg ):
        """Create path based on msg data.

        currently:  (package)/(profile)/(date)_(time)_(session).xml.gz
        """
        filename = (msg.attr['date'] + '_'+ msg.attr['time'] +
                    '_' + msg.attr['session'] +  '.xml.gz')
        relpath = os.path.join( msg.attr['package'], msg.attr['profile'], filename )
        return relpath
        
    def insert(self, msg):
        """Insert new message into repository"""
        packagedir = os.path.join(self.root, msg.attr['package'] )
        if not os.path.isdir( packagedir ):
            os.mkdir( packagedir )
        profiledir = os.path.join( packagedir, msg.attr['profile'] )
        if not os.path.isdir( profiledir ):
            os.mkdir( profiledir )
        relpath = self.createpath( msg ) 
        fullpath = os.path.join ( self.root, relpath )
        gz = gzip.open( fullpath, 'wb')
        gz.write( msg.msg )
        gz.close()
        entry = msg.attr.copy()
        entry['path']=relpath
        self.index.append( entry )
            
        



#
# File:          SortingAlgorithm_Impl.py
# Symbol:        sort.SortingAlgorithm-v0.1
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class sort.SortingAlgorithm in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""An abstract sorting algorithm.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sort.Comparator
import sort.Container
import sort.Counter
import sort.SortingAlgorithm
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
import sort.SimpleCounter
# DO-NOT-DELETE splicer.end(_before_type)

class SortingAlgorithm:
  """\
An abstract sorting algorithm.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = sort.SortingAlgorithm.SortingAlgorithm(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    self.d_swp = sort.SimpleCounter.SimpleCounter()
    self.d_cmp = sort.SimpleCounter.SimpleCounter()
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def getCompareCounter(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # sort.Counter _return
    #

    """\
Return the comparison counter.
"""
# DO-NOT-DELETE splicer.begin(getCompareCounter)
    return self.d_cmp
# DO-NOT-DELETE splicer.end(getCompareCounter)

  def getSwapCounter(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # sort.Counter _return
    #

    """\
Return the swap counter.
"""
# DO-NOT-DELETE splicer.begin(getSwapCounter)
    return self.d_swp
# DO-NOT-DELETE splicer.end(getSwapCounter)

  def reset(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Reset the comparison and swap counter.
"""
# DO-NOT-DELETE splicer.begin(reset)
    self.d_swp.reset()
    self.d_cmp.reset()
# DO-NOT-DELETE splicer.end(reset)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

"""
Test result based solely on the exit status of a process.
"""


import gantlet.events
from gantlet.result_base import result_base, Part

class exit_status_result(result_base):
    """
    Test result based solely on the exit status of a process.
    """
    def __init__( self, raw_name, scoreboard, type, exit_status ):
        """
        create, parse, and store result of a process

        raw_name - the unmodified argument
        scoreboard - the communication hub for events
        type -
        exit_status - the integer exit status
        """
        result_base.__init__( self, raw_name, scoreboard, type )
        self.name = "%s (%s)" % ( self.raw_name, "exit" )
        self.exit_status = exit_status
        # declare the test
        self.number = self.scoreboard.get_id()
        self.declared_nparts = 1
        self.n_parts = 1
        self.scoreboard.event['test_begin'].publish( gantlet.events.test_begin_event, self )
        # create a single part
        partno = 1
        part = Part()        
        part.number = partno
        self.part.append(part)
        self.scoreboard.event['part_begin'].publish( gantlet.events.part_begin_event, (self, partno) )
        # declare the result
        result = ''
        if self.exit_status == None:
            result = 'PASS'
        elif self.exit_status == 77:
            result = 'XFAIL'
        else:
            result = 'FAIL'
        part.result = result
        self.scoreboard.event['part_end'].publish( gantlet.events.part_end_event, (self, partno) )
        # now deal with the end
        self.declared_result = result
        self.PTC_confirm_nparts()
        self.PTC_confirm_result()
        self.scoreboard.event['test_end'].publish( gantlet.events.test_end_event, self )



#
# File:          Employee_Impl.py
# Symbol:        objarg.Employee-v0.5
# Symbol Type:   class
# Babel Version: 1.4.0 (Revision: 6563M trunk)
# Description:   Implementation of sidl class objarg.Employee in Python.
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""This object type holds the basic information about an employee:
name, age, salary and marital status.  This object exists purely
to serve as a test case for sidl.  It is not intended for serious
use.
"""

# DO-NOT-DELETE splicer.begin(_initial)
# Put your code here...
# DO-NOT-DELETE splicer.end(_initial)

import objarg.Employee
import sidl.BaseClass
import sidl.BaseInterface
import sidl.ClassInfo
import sidl.RuntimeException
import sidl.NotImplementedException

# DO-NOT-DELETE splicer.begin(_before_type)
# Put your code here...
# DO-NOT-DELETE splicer.end(_before_type)

class Employee:
  """\
This object type holds the basic information about an employee:
name, age, salary and marital status.  This object exists purely
to serve as a test case for sidl.  It is not intended for serious
use.
"""

# All calls to sidl methods should use __IORself

# Normal Babel creation pases in an IORself. If IORself == None
# that means this Impl class is being constructed for native delegation
  def __init__(self, IORself = None):
    if (IORself == None):
      self.__IORself = objarg.Employee.Employee(impl = self)
    else:
      self.__IORself = IORself
# DO-NOT-DELETE splicer.begin(__init__)
    self.name = None
    self.salary = None
    self.status = None
    self.age = None
# DO-NOT-DELETE splicer.end(__init__)

# Returns the IORself (client stub) of the Impl, mainly for use
# with native delegation
  def _getStub(self):
    return self.__IORself

  def init(self, name, age, salary, status):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # string name
    # int age
    # float salary
    # char status
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # bool _return
    #

    """\
Provide the data for the employee object to hold.
<code>false</code> is returned when the data was unacceptable.
<code>true</code> means the employee object was successfully
initialized.
"""
# DO-NOT-DELETE splicer.begin(init)
    self.name = name
    self.age = age
    self.salary = salary
    self.status = status
    return 1
# DO-NOT-DELETE splicer.end(init)

  def setName(self, name):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # string name
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Change the name of an employee.
"""
# DO-NOT-DELETE splicer.begin(setName)
    self.name = name
# DO-NOT-DELETE splicer.end(setName)

  def getName(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # string _return
    #

    """\
Return the name of an employee.
"""
# DO-NOT-DELETE splicer.begin(getName)
    return self.name
# DO-NOT-DELETE splicer.end(getName)

  def setAge(self, age):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # int age
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Change the age of an employee.
"""
# DO-NOT-DELETE splicer.begin(setAge)
    self.age = age
# DO-NOT-DELETE splicer.end(setAge)

  def getAge(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # int _return
    #

    """\
Return the age of an employee.
"""
# DO-NOT-DELETE splicer.begin(getAge)
    return self.age
# DO-NOT-DELETE splicer.end(getAge)

  def setSalary(self, salary):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # float salary
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Set an employee's salary.
"""
# DO-NOT-DELETE splicer.begin(setSalary)
    self.salary = salary
# DO-NOT-DELETE splicer.end(setSalary)

  def getSalary(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # float _return
    #

    """\
Return an employee's salary.
"""
# DO-NOT-DELETE splicer.begin(getSalary)
    return self.salary
# DO-NOT-DELETE splicer.end(getSalary)

  def setStatus(self, status):
    #
    # sidl EXPECTED INCOMING TYPES
    # ============================
    # char status
    #

    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # None
    #

    """\
Set an employee's marital status.
"""
# DO-NOT-DELETE splicer.begin(setStatus)
    self.status = status
# DO-NOT-DELETE splicer.end(setStatus)

  def getStatus(self):
    #
    # sidl EXPECTED RETURN VALUE(s)
    # =============================
    # char _return
    #

    """\
Return an employee's marital status.
"""
# DO-NOT-DELETE splicer.begin(getStatus)
    return self.status
# DO-NOT-DELETE splicer.end(getStatus)

# DO-NOT-DELETE splicer.begin(_final)
# Put your code here...
# DO-NOT-DELETE splicer.end(_final)



#
# File:          __init__.py
# Symbol:        sidlx-v0.1
# Symbol Type:   package
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Description:   package initialization code
# 
# WARNING: Automatically generated; changes will be lost
# 
#


"""This package contains experimental extensions to sidl
and should not be used unless willing to tolerate its
disappearance in a following release
"""

__all__ = [
   "rmi" ]

try:
  from pkgutil import extend_path
  __path__ = extend_path(__path__, __name__)
except: # ignore all exceptions
  pass



#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.
from Tkinter import *

class ProgressMeter(Frame):
    def __init__( self, parent=None, w=500, h=20, max=100, bgcolor='grey', fgcolor='green'):
        self.bd = 2
        self.width = w
        self.height = h
        self.max = max
        self.color = fgcolor
        Frame.__init__( self, parent )
        self.pack()
        self.canvas = Canvas( self, relief=GROOVE, bg=bgcolor,bd=self.bd )
        self.canvas.pack()
        self.canvas.config(width=self.width-self.bd, height=self.height-self.bd)
        self.completed = 0
        self.meter = self.canvas.create_rectangle(0,0,0, self.height,fill=self.color)
        self.text = self.canvas.create_text(self.width/2, 2*self.height/3, text='0 %',
                                            font=('courier',12,'bold'))


    def updateCompleted( self, completed ):
        #prevent overflow
        completed = max(0, completed)
        completed = min( self.max, completed)
        #calculate new rectangle
        self.completed = completed
        percent_done = float(self.completed) / self.max
        x = int(percent_done * self.width)
        new_meter = self.canvas.create_rectangle(0,0,x,self.height,fill=self.color)
        self.canvas.delete(self.meter)        
        self.meter = new_meter
        newtext = "%d %%" % int( percent_done * 100 )
        self.canvas.itemconfigure( self.text, text=newtext)
        self.canvas.tkraise(self.text)

    def add( self, toadd ):
        self.updateCompleted(self.completed + toadd )

    def subtract( self, tosubtract ):
        self.updateCompleted(self.completed + tosubtract )

    def newMaximum( self, max ):
        self.max = max
        self.updateCompleted( self.completed )

    def tenMoreCompleted( self ):
        self.updateCompleted( self.completed + 10 )

    def tenLessCompleted( self ):
        self.updateCompleted( self.completed - 10 )

    def setGraphically( self, evt ):
        x = evt.x
        percent_done = float(x) / self.width
        completed = int( percent_done * self.max )
        self.updateCompleted( completed )
        
       
if __name__ == '__main__':
    root = Tk()
    pg = ProgressMeter(root)
    Button(root, text='+10', command=pg.tenMoreCompleted ).pack()
    Button(root, text='-10', command=pg.tenLessCompleted ).pack()  
    pg.canvas.bind('<ButtonPress-1>', pg.setGraphically )    
    root.mainloop()
      




#
# Copyright(c) 2001-2003, Lawrence Livermore National Security, LLC.
# Produced at the Lawrence Livermore National Laboratory.
# Written by Gary Kumfert <kumfert@llnl.gov>.
# UCRL-CODE-2002-043.
# All rights reserved.
#
# This file is part of Gantlet.  For details, see 
# http://www.llnl.gov/CASC/components/software.html or contact the author.
#
# Gantlet is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License (as published by 
# the Free Software Foundation) version 2.1 dated February 1999.
#
# Gantlet is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even IMPLIED WARRANTY OF MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the terms and conditions of
# the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ADDITIONAL NOTICE:
# 
# A. This notice is required to be provided under our contract with the
#    U.S. Department of Energy (DOE).  This work was produced at the 
#    University of California, Lawrence Livermore National Laboratory
#    under Contract No. W-7405-ENG-48 with the DOE.
#
# B. Neither the United States Government nor the University of California
#    nor any of their employees make any warranty, express or implied, or
#    assumes any liability or responsibility for the accuracy, completeness, 
#    or usefulness of any information, apparatus, product, or process 
#    disclosed, or represents that its use would not infringe on 
#    privately-owned rights.
# 
# C. Also, reference herein to any specific commercial products, process, or
#    services by trade name, trademark, manufacturer or otherwise does not
#    necessarily constitute or imply its endoresement, recommendation, or
#    favoring by the United States Government or the University of California.
#    The views and opinions of authors expressed herein do not necessarily 
#    state or reflect those of the United States Government or the University
#    of California, and shall not be used for advertising or product 
#    endorsement purposes.

"""Contains the classic_display class.
"""

from gantlet.events import *

class classic_display  :
    """Subscribes to scoreboard events and pretty-prints them to a file.

    Many of the class methods are subscribed to the
    gantlet.scoreboard.scoreboard by __init__ in a publish-subscribe
    pattern.  The different kinds of events are defined in gantlet.events
    """
    
    def __init__( self, scoreboard, file ):
        """
        scoreboard - an instance of gantlet.scoreboard.scoreboard
        file       - a file to write to
        """
        self.linewidth = scoreboard.config.linewidth - 10
        self.file = file
        scoreboard.event['test_begin'].subscribe(
            test_begin_event( self.display_test ) )
        scoreboard.event['test_end'].subscribe(
            test_end_event( self.display_result ) )
        scoreboard.event['suite_finish'].subscribe(
            suite_finish_event( self.display_final ) )
        scoreboard.event['test_error'].subscribe(
            test_error_event( self.display_error ) )
        scoreboard.event['test_warning'].subscribe(
            test_warning_event( self.display_warning ) )
        # scoreboard.event['part_begin'].subscribe(
        #      part_begin_event( self.display_part ) )
        # scoreboard.event['part_end'].subscribe(
        #      part_end_event( self.display_partend ) )
        return
    
    def display_test( self, test ):
        """subscribed to test_begin events"""
        line = "%3d. %s " % (test.number, test.name)
        dots = self.linewidth - len(line)
        self.file.write( line + '.'*dots )
        self.file.flush()
        return 
    
    def display_result( self, test ):
        """subscribed to test_result events"""
        self.file.write( "%7s\n" % test.result)
        return
        
    def display_part( self, test ):
        """subscribed to part_begin events (currently disabled)"""
        print "** part begin %d" % test.part[-1].number
        return 

    def display_partend( self, data ):
        """subscribed to part_end events (currently disabled)"""
        test = data[0]
        resultno = data[1]
        print "** part end %d %s" % (resultno,test.part[-1].result)
        return

    def display_error( self, data ):
        """subscribed to test_error events"""
        test = data[0]
        msg = data[1]
        i = msg.rfind('\n')   # no \n ? then i = -1
        width = len(msg[i+1:])# this works for i==-1 too
        spaces = max(self.linewidth-width-12,0)* ' '
        self.file.write("\n     ERROR: %s%s" % ( msg, spaces ))
        self.file.flush()
        return

    def display_warning( self, data ):
        """subscribed to test_warning events"""
        test = data[0]
        msg = data[1]
        i = msg.rfind('\n')   # no \n ? then i = -1
        width = len(msg[i+1:])# this works for i==-1 too
        spaces = max(self.linewidth-width-14,0)* ' '
        self.file.write("\n     WARNING: %s%s" % ( msg, spaces ))
        self.file.flush()
        return
        
    def display_final( self, scoreboard ):
        """subscribed to suite_finish events"""
        scoreboard.elapsedtime = scoreboard.endtime - scoreboard.starttime
        linebreak = (scoreboard.config.linewidth-1) * '*' 
        header = """
%(date_text)s  at  %(shorttime)s 
by %(whoami)s@%(hostname)s
Wallclock Time: %(elapsedtime)g secs
            Total      Passed      Xfailed      Failed      Broken
Tests     %(tot_tests)7d     %(tot_tests_passed)7d      %(tot_tests_xfailed)7d     %(tot_tests_failed)7d     %(tot_tests_broken)7d
Parts     %(tot_parts)7d     %(tot_parts_passed)7d      %(tot_parts_xfailed)7d     %(tot_parts_failed)7d
""" % scoreboard.__dict__
        self.file.write(linebreak)
        self.file.write(header)
        self.file.write(linebreak + '\n')
        self.failed_tests(scoreboard)
        return

    def failed_tests( self, scoreboard ):
        """Prints a formatted table of failed tests

        Utility for display_final()"""

        if scoreboard.tot_tests_failed + scoreboard.tot_tests_broken == 0:
            pass
        self.file.write("   Broken|Failed|Warning                      Exit  Tot %Fail List of Failed")
        self.file.write("\n")
        self.file.write('-'*(scoreboard.config.linewidth-1))
        self.file.write("\n")
        for test in scoreboard.tests:
            if test.errmsg != '' or test.n_parts_failed > 0:
                self.print_failed_test( test )
        return


    def print_failed_test( self, test ):
        """Prints a line of data in a tabel of failed tests

        Utility for failed_tests()"""
        number = test.number
        name = test.name
        bfw = 'W'
        status = ''
        if test.result == 'FAIL':
            bfw = 'F'
        elif test.result == 'BROKEN':
            bfw = 'B'
        if test.exit_status:
            status = "%d" % test.exit_status
        total = test.n_parts
        fail = test.n_parts_failed
        failed = 0.0
        if total != 0:
            failed = 100 * fail/total 
        list_of_failed = test.canon
        if test.result != 'BROKEN':
            list_of_failed = list_of_failed + self.get_cannon( test )
        if len( list_of_failed) > 18:
            list_of_failed = list_of_failed[:15] + '...'
        self.file.write("%3d %c %-40s %3s %4d %3d%% %s\n" %
                        (number, bfw, name, status, total, failed, list_of_failed))
        return
        
    def get_cannon( self, test ):
        """returns a formatted string of failed part numbers in test

        Utility for print_failed_test()"""
        start = -1
        current = -1
        canon = ''
        state = 'before runs' # also 'between runs' and 'in runs'
        for part in test.part:
            if part.result == 'FAIL':
                current = part.number
                if state == 'between runs':
                    start = part.number
                    canon = canon + ', %d' % part.number
                    state = 'in run'
                elif state == 'before runs':
                    start = part.number
                    canon = canon + "%d" % part.number
                    state = 'in run'
            else:
                if state == 'between runs' or state == 'before runs':
                    pass
                elif state == 'in run':
                    if current > start: # current 
                        canon = canon + '-%d' % current
                    state = 'between runs'
                else:
                    state = 'between runs'
                current = -1
        if state == 'in run':
            canon = canon + '-%d' % len( test.part )
        return canon
            
